using System;
using NUnit.Framework;
using NUnit.Framework.SyntaxHelpers;
using Rhino.Mocks;
using Spark.Modules;

namespace Modules.RhinoMocks.Tests.FlairTests.BlockTests
{
    [TestFixture]
    public class AbstractClassTests
    {
        private MockRepository mockery;
        private MyAbstractClass abstractClass; 

        [SetUp]
        public void Setup()
        {
            mockery = new MockRepository();
            abstractClass = mockery.PartialMock<MyAbstractClass>(); 
        }

        [Test]
        public void Mocking_a_abstract_and_calling_its_implemented_methods()
        {
            With.Mocks(mockery)
            .Expecting(delegate
            {
                
            })
            .Verify(delegate
            {
                var value = abstractClass.RoundValue(5.65, 1);
                Assert.That(value, Is.EqualTo(5.6));
            });
        }

        [Test]
        public void Mocking_a_abstract_and_calling_its_abstract_methods()
        {
            With.Mocks(mockery)
            .Expecting(delegate
            {
                Expect.Call(abstractClass.DoSomethingAndReturnAString()).Return("Foo");
            })
            .Verify(delegate
            {
                var value = abstractClass.DoSomethingAndReturnAString();
                Assert.That(value, Is.EqualTo("Foo"));
            });
        }


    }

    public abstract class MyAbstractClass
    {
        public abstract string DoSomethingAndReturnAString(); 
       
        public virtual double CalculatePI()
        {
            return Math.PI; 
        }

        public virtual double RoundValue(double value, Int32 digits)
        {
            return Math.Round(value, digits);
        }
    }
}