﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Modular.Flair.WebPackage.Blocks;
using Modular.Flair.WebPackage.Models;
using Modular.Flair.WebPackage.Services;
using NUnit.Framework;
using NUnit.Framework.SyntaxHelpers;
using Rhino.Mocks;

namespace Modules.RhinoMocks.Tests.FlairTests.BlockTests
{
    [TestFixture]
    public class FlairTestsWithContainer : BlockTestBase
    {
        private DidYouKnowBlock block;
        private ITriviaProvider mockTriviaProvider; 

        [SetUp]
        public override void Setup()
        {
            base.Setup();
            kernel.AddComponent<DidYouKnowBlock>();
            kernel.AddComponentInstance<ITriviaProvider>(mockTriviaProvider = MockRepository.GenerateMock<ITriviaProvider>());
            block = kernel.Resolve<DidYouKnowBlock>(); 
        }

        [Test]
        public void The_flair_block_should_use_the_trivia_provider_to_get_its_question_and_load_them_into_the_viewdata()
        {
            // Arrange 
            mockTriviaProvider.Expect(x => x.GetRandomTrivia()).Return(new Trivia {Details = "Foo", Teaser = "Bar"}); 

            // Act
            block.RenderBlock();

            // Assert
            var args = mockHtmlHelper.GetArgumentsForCallsMadeOn(
                x => x.RenderPartial(Arg<string>.Is.Anything, Arg<object>.Is.Anything));

            // Verify the path to the view
            var firstArgument = (string)args.First().First();
            Assert.That(firstArgument, Is.EqualTo(@"Flair\DidYouKnow\Block"));
            

            var secondArgument = (Trivia) args.First()[1];
            // Verify the model details
            Assert.That(secondArgument.Details, Is.EqualTo("Foo"));
            Assert.That(secondArgument.Teaser, Is.EqualTo("Bar"));
        }

    }
}