using System;
using System.Linq;
using System.Web.Mvc;
using Modular.Flair.WebPackage.Blocks;
using Modular.Flair.WebPackage.Models;
using Modular.Flair.WebPackage.Services;
using NUnit.Framework;
using NUnit.Framework.SyntaxHelpers;
using Rhino.Mocks;
using Spark.Modules;

namespace Modules.RhinoMocks.Tests.FlairTests.BlockTests
{
    [TestFixture]
    public class FlairTestsWithoutContainer
    {
        private DidYouKnowBlock block;
        private ITriviaProvider mockTriviaProvider;
        private IHtmlHelperDuck mockHtmlHelper;
        private ViewDataDictionary stubViewData; 

        [SetUp]
        public  void Setup()
        {
            mockTriviaProvider = MockRepository.GenerateMock<ITriviaProvider>();
            mockHtmlHelper = MockRepository.GenerateMock<IHtmlHelperDuck>();
            stubViewData = MockRepository.GenerateStub<ViewDataDictionary>();

            // Set the depenedencies through setter injection.
            block = new DidYouKnowBlock(mockTriviaProvider)
                        {
                            Html = mockHtmlHelper, 
                            ViewData = stubViewData
                        };
        }

        [Test]
        public void The_flair_block_should_use_the_trivia_provider_to_get_its_question_and_load_them_into_the_viewdata()
        {
            // Arrange 
            mockTriviaProvider
                .Expect(x => x.GetRandomTrivia())
                .Return(new Trivia
                            {
                                Details = "Foo",
                                Teaser = "Bar"
                            }); 

            // Act
            block.RenderBlock();

            // Assert
            var callsMadeOn = mockHtmlHelper.GetArgumentsForCallsMadeOn
                (x => 
                 x.RenderPartial(
                    Arg<string>.Is.Anything, 
                    Arg<object>.Is.Anything)
                );


            var firstArgument = (string) callsMadeOn.First().First();
            Assert.That(firstArgument, Is.EqualTo(@"Flair\DidYouKnow\Block"));

            var secondArgument = (Trivia)callsMadeOn.First()[1];

            Assert.That(secondArgument.Details, Is.EqualTo("Foo"));
            Assert.That(secondArgument.Teaser, Is.EqualTo("Bar"));

        }
    }
}